@@ -55,6 +55,8 @@ programa, armazenada no arquivo COPYING).
 #include "editmeta.h"
 #include "keybind.h"
 #include "commands.h"
+#include "menubar.h"
+#include "welcomedlg.h"
 
 #define FAST_MOVE_AMOUNT 8  /* how much the cursor moves each time the fast
                              * movent keys (Ctrl+Z, Ctrl+X) are pressed */
@@ -200,16 +202,20 @@ void u_load_file(const char *supplied_filename) {
 
    if (!supplied_filename) {
       filename = ui_ask_s("Load file", NULL);
-      if (ui_cancel) return;
+      if (ui_cancel || !*filename) {
+         free(filename);
+         return;
+      }
 
-      if(filename[0] == '~') {
-         expanded_filename = expand_tilde(filename);
-         if(!expanded_filename)
+      if (*filename == '~') {
+         if ( !(expanded_filename = expand_tilde(filename)) ) {
+            free(filename);
             return;
+         }
 
          /* Comparing pointers here. If no expansion
           * was made, original filename is returned. */
-         if(expanded_filename != filename) {
+         if (expanded_filename != filename) {
             free(filename);
             filename = expanded_filename;
          }
@@ -333,6 +339,7 @@ static int lgmode_map_ch(int ch) {
 /* handle special keys for SM_SELECT mode */
 static void sm_select_handle_key(int ch) {
    int x0, y0, x1, y1;
+   int tmp;
    get_norm_sel(&x0, &y0, &x1, &y1);
    
    /* we are in SM_SELECT mode */
@@ -358,6 +365,13 @@ static void sm_select_handle_key(int ch) {
       case 'b':
          tint_sel(false);
          break;
+      case 'o':
+         /* swap the anchor with the cursor position *
+          * that will enable the user to manipulate the "other edge"
+          * of the rectangle */
+         tmp = _x; _x = _ax; _ax = tmp; 
+         tmp = _y; _y = _ay; _ay = tmp; 
+         break;
    }
 }
 
@@ -426,6 +440,14 @@ void handle_key(int ch) {
    int command;
    Layer *lyr;
 
+   /* first check for function keys (they open the menu) */
+   if (ch > KEY_F0 && ch <= KEY_F(12)) {
+      /* show menu and perform command mandated by it */
+      command = menubar_show(ch - KEY_F0 - 1);
+      if (command != COMMAND_UNDEFINED) handle_command(command);
+      return;
+   }
+
    if ( (command = keybind_translate(ch)) ) {
       /* key triggers a command: execute command */
       handle_command(command);
@@ -481,28 +503,36 @@ void handle_command(int command) {
    /* handle commands that work regardless of whether document is empty */
    switch (command) {
       case COMMAND_QUIT: exit(0); break;
-      case COMMAND_ADD_LAYER_DEFAULTS: u_add_layer(false); break;
-      case COMMAND_ADD_LAYER_SPECIFY:  u_add_layer(true); break;
+      case COMMAND_ADD_LAYER_DEFAULTS:
+      case COMMAND_ADD_LAYER_SPECIFY:
+         u_add_layer(command == COMMAND_ADD_LAYER_SPECIFY);
+         if (!doc_empty()) _lyr = 0;  /* now we have a layer, so correct _lyr
+                                       * so that it makes sense */
+         return;
       case COMMAND_SET_FOREGROUND:
          newfg = ui_ask_color("Foreground color"); 
          if (!ui_cancel) _fg = newfg;
-         break;
+         return;
       case COMMAND_SET_BACKGROUND:
          newbg = ui_ask_color("Background color");
          if (!ui_cancel) _bg = newbg;
-         break;
-      case COMMAND_SHOW_COLOR_DLG: show_color_dlg(); break;
-      case COMMAND_TOGGLE_INSERT:  _insmode = !_insmode; break;
-      case COMMAND_SHOW_HELP_DLG:  show_help_dlg(); break;
-      case COMMAND_LOAD_FILE: u_load_file(NULL); break;
-      case COMMAND_SAVE_FILE: u_save_file(); break;
-      case COMMAND_EDIT_META: u_edit_meta(); break;
+         return;
+      case COMMAND_SHOW_COLOR_DLG: show_color_dlg(); return;
+      case COMMAND_TOGGLE_INSERT:  _insmode = !_insmode; return;
+      case COMMAND_SHOW_HELP_DLG:  show_help_dlg(); return;
+      case COMMAND_SHOW_ABOUT_DLG: show_welcome_dlg(); return;
+      case COMMAND_LOAD_FILE: u_load_file(NULL); return;
+      case COMMAND_SAVE_FILE: u_save_file(); return;
+      case COMMAND_NEW_FILE:
+         document_destroy(_doc);
+         zero_state();
+         return;
+      case COMMAND_EDIT_META: u_edit_meta(); return;
    }
 
-   if (doc_empty()) return;
-
    /* from this point on, we handle the keys that only work if the document
     * is not empty */
+   if (doc_empty()) return;
    lyr = _doc->layers[_lyr];
   
    /* if clipboard exists and we are not in SM_FLOAT mode, delete it */
